# Form implementation generated from reading ui file 'pyqt6rc/test/myPackage/templates/template1.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from importlib.resources import path

from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(292, 208)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.pushButton = QtWidgets.QPushButton(Form)
        icon = QtGui.QIcon()
        with path("myPackage.resources.icons", "icon1.png") as f_path:
            icon.addPixmap(QtGui.QPixmap(str(f_path)), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.pushButton.setIcon(icon)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout.addWidget(self.pushButton, 0, 0, 1, 1)
        self.pushButton_2 = QtWidgets.QPushButton(Form)
        icon1 = QtGui.QIcon()
        with path("myPackage.resources.icons", "icon2.png") as f_path:
            icon1.addPixmap(QtGui.QPixmap(str(f_path)), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.pushButton_2.setIcon(icon1)
        self.pushButton_2.setObjectName("pushButton_2")
        self.gridLayout.addWidget(self.pushButton_2, 1, 0, 1, 1)
        self.label = QtWidgets.QLabel(Form)
        self.label.setText("")
        with path("myPackage.resources", "image1.png") as f_path:
            self.label.setPixmap(QtGui.QPixmap(str(f_path)))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setText("")
        with path("myPackage.resources", "image2.png") as f_path:
            self.label_2.setPixmap(QtGui.QPixmap(str(f_path)))
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 1, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.pushButton.setText(_translate("Form", "PushButton"))
        self.pushButton_2.setText(_translate("Form", "PushButton"))



import logging
import os
import xml.etree.ElementTree as Et
from os.path import dirname, basename
from typing import List, Optional, Dict

from pyqt6rc import resource_pattern, indent_pattern


def get_module_path(input_dir: str) -> str:
    """
    Get module path by crawling parent folders and looking for __init__.py file.
    Last folder having __init__.py is top-level package name.
    :param str input_dir: input directory
    :return: module path
    """
    package_parts = []
    while True:
        try:
            for entry in os.scandir(input_dir):
                if entry.name == "__init__.py":
                    package_parts.insert(0, os.path.basename(input_dir))
                    input_dir = os.path.dirname(input_dir)
                    break
            else:
                break
        except FileNotFoundError:
            break
    return ".".join(package_parts)


def parse_qrc(qrc_file: str) -> dict:
    """
    Parse qrc xml file and extract prefixes and aliases.
    :param str qrc_file: path to qrc_file
    :return dict: parsed qrc
    """
    tree = Et.parse(qrc_file)
    root = tree.getroot()

    if root.tag != "RCC":
        raise Exception(f"Invalid Resource file format.")

    resources = {}
    for child in root:
        if child.tag == "qresource":
            aliases = {}
            for file_child in child:
                alias = file_child.attrib.get("alias", None)
                if alias is not None:
                    aliases[file_child.text] = alias
            prefix = "/" + child.attrib.get("prefix", "")
            if not prefix.endswith("/"):
                prefix += "/"

            resources[prefix] = {
                "module_path": get_module_path(os.path.dirname(qrc_file)),
                "aliases": aliases
            }
    return resources


def update_resources(ui_file: str, resources: Dict) -> Optional[str]:
    """
    Read ui file and collect all input resource files.
    :param str ui_file: input ui template
    :param dict resources: input parsed resources
    :return:
    """
    tree = Et.parse(ui_file)
    root = tree.getroot()

    if root.tag != "ui":
        raise Exception(f"Invalid template file format.")

    ui_dir = os.path.dirname(ui_file)
    location = None
    for child in root:
        if child.tag == "resources":
            for include in child:
                location = include.attrib.get("location", None)
                if location is not None:
                    resource_location = os.path.normpath(os.path.join(ui_dir, location))
                    resources.update(parse_qrc(resource_location))
    return dirname(location) if location is not None else None


def ui_to_py(ui_file: str) -> str:
    """
    Use pyuic6 to convert ui template into py file
    :param str ui_file: input ui template file
    :return str: converted python template
    """
    return os.popen(f"pyuic6 {ui_file}").read()


def modify_py(py_input: str, resources: dict, tab_size: int = 4, compatible: bool = False) -> str:
    """
    Modify python template, wrap resource files with path(resource_package, f_name) as f_path.
    :param str py_input: converted python template
    :param dict resources: collected resources
    :param int tab_size: number of spaces in one tab
    :param bool compatible: use compatible importlib_resources instead of native importlib
    :return str: modified python template
    """
    output = ""
    imported = False
    tab = " " * tab_size

    for line in py_input.split("\n"):
        # Check if path was imported
        if not imported and line.startswith("from"):
            if compatible:
                output += "from importlib_resources import path\n"
            else:
                output += "from importlib.resources import path\n"
            imported = True
        # Check if any resource path is in line
        out = resource_pattern.search(line)
        if out is not None:
            tabs = indent_pattern.search(line)
            package, path = None, None
            # Check if resource path starts with any of the prefixes
            for prefix in resources.keys():
                if out[1].startswith(prefix):
                    # make file path by removing prefix from it
                    package = resources[prefix]["module_path"]
                    path = out[1][len(prefix):]
                    break
            if path is None:
                # Prefix doesn't exist in qrc file, comment out that line
                logging.warning(f"Prefix \"{out[1].split('/')[1]}\" not found in qrc file.")
                output += "# " + line + "\n"
                continue

            # Split file path into parts
            path_parts = list(filter(None, dirname(path).split("/")))
            # Make final resource_package name
            resource_package = ".".join([package] + path_parts)
            # Get file name
            f_name = basename(out[1])
            output += f"{tabs[0]}with path(\"{resource_package}\", \"{f_name}\") as f_path:\n"
            line = tab + line.replace(out[0], f"str(f_path)")

        # Append new line into output
        output += line + "\n"
    return output


def modify_py_sp(py_input: str, resources: dict, resource_rel_path: str, tab_size: int = 4) -> str:
    """
    Modify python template, wrap resource files with path(resource_package, f_name) as f_path.
    :param str py_input: converted python template
    :param dict resources: collected resources
    :param str resource_rel_path: Relative path to the resource
    :param int tab_size: number of spaces in one tab
    :return str: modified python template
    """
    output = ""
    imported = False
    def_placeholder = False
    tab = " " * tab_size
    prefix_resources = set()
    placeholder = f"#{tab}__PLACEHOLDER__"
    if not resources:
        return py_input
    for index, line in enumerate(py_input.split("\n")):
        # Check if path was imported
        if not imported and line.startswith("from"):
            # Import all required packages
            output += "import os\n" \
                      "from os.path import dirname, normpath\n" \
                      "from PyQt6.QtCore import QDir\n"
            imported = True
        elif not def_placeholder and line.startswith(f"{tab}def setupUi"):
            # Append placeholder after setupUi definition
            line = line + "\n" + placeholder
            def_placeholder = True
        else:
            # Check if any resource path is in line
            out = resource_pattern.search(line)
            if out is not None:
                path, prefix = None, None
                # Check if resource path starts with any of the prefixes
                for prefix in resources.keys():
                    if out[1].startswith(prefix):
                        # make file path by removing prefix from it
                        path = out[1][len(prefix):]
                        break
                if path is None or prefix is None:
                    # Prefix doesn't exist in qrc file, comment out that line
                    logging.warning(f"Prefix \"{out[1].split('/')[1]}\" not found in qrc file.")
                    output += "# " + line + "\n"
                    continue

                # Add prefix resource in format (prefix, resource_rel_path/filepath)
                prefix_resources.add((prefix[1:-1], "/".join([resource_rel_path, dirname(path)])))
                # This creates file reference in format prefix:filename
                f_path = f"{prefix[1:-1]}:{basename(out[1])}"
                # Replace previous file reference of modified one
                line = line.replace(out[0], f'"{f_path}"')

        # Append new line into output
        output += line + "\n"
    # Generate code which setSearchPath for every prefix:resource
    # To get absolute dir path, combine current __filename__ dir, resource relative path and normalize it
    append_path_part = f'{tab * 2}prefix_resources = {list(prefix_resources)}\n' \
                       f'{tab * 2}for prefix, resource in prefix_resources:\n' \
                       f'{tab * 3}sp = QDir.searchPaths(prefix)\n' \
                       f'{tab * 3}QDir.setSearchPaths(prefix, set(sp + [normpath(os.path.join(dirname(__file__), resource))]))\n'
    output = output.replace(placeholder, append_path_part, 1)
    return output


def save_py(ui_file: str, py_input: str, output_dir: Optional[str] = None) -> None:
    """
    Save python template into file.
    Use ui filename and change .ui suffix to .py.
    If output_dir is None, use same dir as a .ui template file to store converted .py template.
    :param str ui_file: input ui template file
    :param str py_input: converted python template
    :param str output_dir: output directory
    :return: None
    """
    input_filename = os.path.basename(ui_file)
    parts = input_filename.split(".")
    parts[-1] = "py"

    if output_dir is None:
        output_dir = os.path.dirname(ui_file)
    else:
        if not os.path.exists(output_dir):
            os.mkdir(output_dir)

    output_filename = ".".join(parts)
    output_filename_path = os.path.join(output_dir, output_filename)
    with open(output_filename_path, "w") as fp:
        fp.write(py_input)
    logging.info(f"{input_filename} > {output_filename}")


def get_ui_files(input_dir: str) -> List[str]:
    """
    Get all .ui template files in input directory
    :param str input_dir: input directory
    :return list: list of .ui files
    """
    files = []
    for entry in os.scandir(input_dir):
        if entry.is_file(follow_symlinks=False) and entry.name.endswith(".ui"):
            files.append(entry.path)
    logging.info(f"Found {len(files)} .ui files")
    return files



# Form implementation generated from reading ui file '/home/domarm/PycharmProjects/pyqt6rc/pyqt6rc/test/myPackage/templates/template3.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(400, 300)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))



# Form implementation generated from reading ui file '/home/domarm/PycharmProjects/pyqt6rc/pyqt6rc/test/myPackage/templates/template2.ui'
#
# Created by: PyQt6 UI code generator 6.1.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


import os
from os.path import dirname, normpath

from PyQt6 import QtCore, QtGui, QtWidgets
from PyQt6.QtCore import QDir


class Ui_Form(object):
    def setupUi(self, Form):
        prefix_resources = [('icons', '../resources/icons'), ('other_images', '../resources/'),
                            ('other_icons', '../resources/icons'), ('images', '../resources/')]
        for prefix, resource in prefix_resources:
            sp = QDir.searchPaths(prefix)
            QDir.setSearchPaths(prefix, set(sp + [normpath(os.path.join(dirname(__file__), resource))]))

        Form.setObjectName("Form")
        Form.resize(292, 208)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(Form)
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap("images:image1.png"))
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 2, 0, 1, 1)
        self.pushButton_3 = QtWidgets.QPushButton(Form)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("other_images:image1_r2.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.pushButton_3.setIcon(icon)
        self.pushButton_3.setObjectName("pushButton_3")
        self.gridLayout.addWidget(self.pushButton_3, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setText("")
        #         self.label_2.setPixmap(QtGui.QPixmap(":/images_non_existent/image2.png"))
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 2, 1, 1, 1)
        self.pushButton_4 = QtWidgets.QPushButton(Form)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap("other_icons:icon2_r2.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        icon1.addPixmap(QtGui.QPixmap("icons:icon1.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.On)
        self.pushButton_4.setIcon(icon1)
        self.pushButton_4.setObjectName("pushButton_4")
        self.gridLayout.addWidget(self.pushButton_4, 1, 1, 1, 1)
        self.pushButton = QtWidgets.QPushButton(Form)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap("icons:icon1.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.pushButton.setIcon(icon2)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout.addWidget(self.pushButton, 0, 0, 1, 1)
        self.pushButton_2 = QtWidgets.QPushButton(Form)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap("icons:icon2.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        self.pushButton_2.setIcon(icon3)
        self.pushButton_2.setObjectName("pushButton_2")
        self.gridLayout.addWidget(self.pushButton_2, 1, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setText("")
        self.label_3.setPixmap(QtGui.QPixmap("other_images:image1_r2.png"))
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 3, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setText("")
        self.label_4.setPixmap(QtGui.QPixmap("other_images:image2_r2.png"))
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 3, 1, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.pushButton_3.setText(_translate("Form", "PushButton"))
        self.pushButton_4.setText(_translate("Form", "PushButton"))
        self.pushButton.setText(_translate("Form", "PushButton"))
        self.pushButton_2.setText(_translate("Form", "PushButton"))



import logging


def set_logger(disabled=False):
    logger = logging.getLogger()
    sh = logging.StreamHandler()
    formatter = logging.Formatter('%(levelname)s - %(message)s')
    sh.setFormatter(formatter)
    logger.setLevel(logging.INFO)
    logger.addHandler(sh)
    logger.disabled = disabled



import os.path

import pytest

from pyqt6rc.convert_tools import parse_qrc, ui_to_py, modify_py, get_ui_files, save_py, update_resources, modify_py_sp


def test_qrc_parse() -> None:
    parsed_qrc = parse_qrc("pyqt6rc/test/test_resources/resources.qrc")
    assert parsed_qrc == {'/': {'aliases': {}, 'module_path': ''},
                          '/icons/': {'aliases': {}, 'module_path': ''},
                          '/subdir/icons/': {'aliases': {}, 'module_path': ''},
                          '/subdir/icons_aliased/': {'aliases': {'test.png': 'aliased.png'},
                                                     'module_path': ''}}


def test_broken_qrc_parse() -> None:
    with pytest.raises(Exception):
        parse_qrc("pyqt6rc/test/test_resources/broken_resources.qrc")


@pytest.mark.parametrize("compatible", [False, True])
def test_conversion(compatible) -> None:
    reference_file = "template1_reference_compatible.py" if compatible else "template1_reference.py"

    resources = {}
    update_resources("pyqt6rc/test/test_resources/myPackage/templates/template1.ui", resources)
    convert_ui_to_py = ui_to_py("pyqt6rc/test/test_resources/myPackage/templates/template1.ui")
    modified_py = modify_py(convert_ui_to_py, resources, compatible=compatible)

    with open(f"pyqt6rc/test/test_resources/myPackage/templates/{reference_file}", "r") as fp:
        assert fp.read().split("\n", 6)[2] == modified_py.split("\n", 6)[2]


def test_sp_conversion() -> None:
    reference_file = "template1_sp_reference.py"

    resources = {}
    resource_rel_path = update_resources("pyqt6rc/test/test_resources/myPackage/templates/template1.ui", resources)
    convert_ui_to_py = ui_to_py("pyqt6rc/test/test_resources/myPackage/templates/template1.ui")
    modified_py = modify_py_sp(convert_ui_to_py, resources, resource_rel_path)

    with open(f"pyqt6rc/test/test_resources/myPackage/templates/{reference_file}", "r") as fp:
        assert fp.read().split("\n", 6)[2] == modified_py.split("\n", 6)[2]


def test_sp_conversion_no_resources() -> None:
    reference_file = "template3_reference.py"

    resources = {}
    resource_rel_path = update_resources("pyqt6rc/test/test_resources/myPackage/templates/template3.ui", resources)
    convert_ui_to_py = ui_to_py("pyqt6rc/test/test_resources/myPackage/templates/template3.ui")
    modified_py = modify_py_sp(convert_ui_to_py, resources, resource_rel_path)

    with open(f"pyqt6rc/test/test_resources/myPackage/templates/{reference_file}", "r") as fp:
        assert fp.read().split("\n", 6)[2] == modified_py.split("\n", 6)[2]


def test_get_ui_files():
    ui_files = get_ui_files("pyqt6rc/test/test_resources/myPackage/templates")
    assert 'pyqt6rc/test/test_resources/myPackage/templates/template3.ui' in ui_files
    assert 'pyqt6rc/test/test_resources/myPackage/templates/template2.ui' in ui_files
    assert 'pyqt6rc/test/test_resources/myPackage/templates/template1.ui' in ui_files


def test_save_py():
    parsed_qrc = parse_qrc("pyqt6rc/test/test_resources/myPackage/resources/resources.qrc")
    for template_name in ("template1", "template2"):
        convert_ui_to_py = ui_to_py(f"pyqt6rc/test/test_resources/myPackage/templates/{template_name}.ui")
        modified_py = modify_py(convert_ui_to_py, parsed_qrc)

        save_py(f"pyqt6rc/test/test_resources/myPackage/templates/{template_name}.ui", modified_py, "pyqt6rc/test/")
        assert os.path.isfile(f"pyqt6rc/test/{template_name}.py")

        try:
            with open(f"pyqt6rc/test/{template_name}.py", "r") as fp:
                assert fp.read().split("\n", 6)[2] == modified_py.split("\n", 6)[2]
        finally:
            os.remove(f"pyqt6rc/test/{template_name}.py")



import logging

import pytest

from pyqt6rc.script_helpers import set_logger


@pytest.mark.parametrize("disabled", [False, True])
def test_set_logger(disabled, caplog, request) -> None:
    test_id = request.node.callspec.id
    set_logger(disabled)
    logging.info("test")
    if test_id == "True":
        assert caplog.messages == []
    else:
        assert caplog.messages == ["test"]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyqt6rc",
    version="0.4.0",
    license='MIT',
    author="Martin Domaracký",
    author_email="domarm@comat.sk",
    description="PyQt6 UI templates resource converter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/domarm-comat/pyqt6rc",
    packages=setuptools.find_packages(),
    package_data={
        'pyqt6rc.scripts': ['pyqt6rc', 'pyqt6sp'],
        'pyqt6rc.test': ["*/*", "test_resources/myPackage/*", "test_resources/myPackage/*/*",
                         "test_resources/myPackage/*/*/*"],
    },
    scripts=['pyqt6rc/scripts/pyqt6rc', 'pyqt6rc/scripts/pyqt6sp'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Intended Audience :: Science/Research",
    ],
    install_requires=[
        "pyqt6",
        "pyqt6-tools",
        "importlib_resources"
    ],
    python_requires='>=3.7',
)
